from bs4 import BeautifulSoup
from ..extractor import Extractor


class WechatGZHExtractor(Extractor):
    """
    微信公众号
    """

    def __init__(self):
        super().__init__()
        self.attrs_to_clean.append(lambda el: 'id' in el.attrs and el.attrs['id'] == 'meta_content')

    def can_handle(self, soup: BeautifulSoup) -> bool:
        return self.get_og_site_name(soup) == "微信公众平台"

    def article_container(self) -> tuple:
        return ("div", {"class": "rich_media_content"})



from bs4 import BeautifulSoup
from ..extractor import Extractor


class QuantamagazineExtractor(Extractor):
    """
    quantamagazine.org
    """

    def __init__(self):
        super().__init__()
        self.attrs_to_clean.extend([
            lambda el: 'class' in el.attrs and 'post__title__title' in el.attrs['class'],
        ])

    def can_handle(self, soup: BeautifulSoup) -> bool:
        return self.get_og_site_name(soup) == "Quanta Magazine"

    def article_container(self) -> tuple:
        return ("div", {"id": "postBody"})



from bs4 import BeautifulSoup
from ..extractor import Extractor


class InfoQCNExtractor(Extractor):
    """
    www.infoq.cn
    """

    def can_handle(self, soup: BeautifulSoup) -> bool:
        canonical_tag = soup.find("link", {"rel": "canonical"})
        canonical = canonical_tag["href"].strip() if canonical_tag and canonical_tag.has_attr("href") else None
        return canonical and canonical.startswith("https://www.infoq.cn")

    def article_container(self) -> tuple:
        return ("div", {"class": "article-content-wrap"})



from typing import Protocol

from .hookspecs import ReaderPlugin
from .plugins import pm
from .utils import REQUEST_HEADERS

import requests


class Reader(Protocol):
    def read(self) -> str:
        ...


class ReaderFactory:
    @staticmethod
    def create(url_or_path: str) -> Reader:
        custom_plugin_reader = pm.hook.get_custom_reader(url=url_or_path)
        if custom_plugin_reader:
            class PluginReaderAdapter(Reader):
                def __init__(self, plugin: ReaderPlugin, url: str):
                    self.plugin = plugin
                    self.url = url
                def read(self) -> str:
                    return self.plugin.read(self.url)
            return PluginReaderAdapter(custom_plugin_reader, url_or_path)
        if url_or_path.startswith("http"):
            return HtmlReader(url_or_path)
        else:
            return FileReader(url_or_path)


class HtmlReader(Reader):
    def __init__(self, url_or_path: str):
        self.url_or_path = url_or_path

    def read(self) -> str:
        response = requests.get(self.url_or_path, headers=REQUEST_HEADERS)
        response.encoding = "utf-8"
        return response.text


class FileReader(Reader):
    def __init__(self, url_or_path: str):
        self.url_or_path = url_or_path

    def read(self) -> str:
        with open(self.url_or_path, "r", encoding="utf8") as f:
            return f.read()



from .omni_article_md import OmniArticleMarkdown

__all__ = ['OmniArticleMarkdown']

DEFAULT_PLUGINS = {
    "zhihu": "omnimd-zhihu-reader",
    "freedium": "omnimd-freedium-reader",
    "toutiao": "omnimd-toutiao-reader",
    "appledev": "omnimd-appledev-reader",
}



from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import Optional
from bs4 import BeautifulSoup, element

from .utils import Constants, extract_article_from_soup


@dataclass
class Article:
    title: str
    description: str
    body: element.Tag


class Extractor(ABC):

    def __init__(self):
        self.tags_to_clean = Constants.TAGS_TO_CLEAN
        self.attrs_to_clean = Constants.ATTRS_TO_CLEAN

    def extract(self, soup: BeautifulSoup) -> Optional[Article]:
        if self.can_handle(soup):
            article_container = self.article_container()
            if isinstance(article_container, tuple):
                article_container = [article_container]
            for container in article_container:
                article_tag: element.Tag = extract_article_from_soup(soup, container)
                if article_tag:
                    # print(f"Using extractor: {self.__class__.__name__}")
                    h1 = article_tag.find("h1")
                    if h1:
                        h1.decompose()
                    for tag in article_tag.find_all():
                        if any(cond(tag) for cond in self.tags_to_clean):
                            tag.decompose()
                            continue
                        if tag.attrs:
                            if any(cond(tag) for cond in self.attrs_to_clean):
                                tag.decompose()
                    title = self.extract_title(soup)
                    description = self.extract_description(soup)
                    return Article(title=title, description=description, body=article_tag)
        return None

    @abstractmethod
    def can_handle(self, soup: BeautifulSoup) -> bool:
        ...

    @abstractmethod
    def article_container(self) -> tuple | list:
        ...

    def extract_title(self, soup: BeautifulSoup) -> str:
        og_title_tag = soup.find("meta", {"property": "og:title"})
        title = og_title_tag["content"].strip() if og_title_tag and "content" in og_title_tag.attrs else None
        if title:
            return title
        title_tag = soup.title
        title = title_tag.text.strip() if title_tag else None
        # 确保 title 不为 None
        return title or "Untitled"

    def extract_description(self, soup: BeautifulSoup) -> Optional[str]:
        og_desc_tag = soup.find("meta", {"property": "og:description"})
        return og_desc_tag["content"].strip() if og_desc_tag and "content" in og_desc_tag.attrs else None

    def get_og_site_name(self, soup: BeautifulSoup) -> Optional[str]:
        site_name_tag = soup.find("meta", {"property": "og:site_name"})
        return site_name_tag["content"].strip() if site_name_tag and site_name_tag.has_attr("content") else None

class DefaultExtractor(Extractor):
    def can_handle(self, soup: BeautifulSoup) -> bool:
        return True

    def article_container(self) -> tuple | list:
        return Constants.ARTICLE_CONTAINERS



from pathlib import Path

from .readers import ReaderFactory
from .html_md_parser import HtmlMarkdownParser
from .utils import to_snake_case


class OmniArticleMarkdown:

    DEFAULT_SAVE_PATH = "./"

    def __init__(self, url_or_path: str):
        self.url_or_path = url_or_path
        self.title = None
        self.markdown = None
        self.save_path = None

    def parse(self) -> str:
        reader = ReaderFactory.create(self.url_or_path)
        raw_html = reader.read()
        html_arser = HtmlMarkdownParser(raw_html)
        self.title, self.markdown = html_arser.parse()
        return self.markdown

    def save(self, save_path: str = None):
        save_path = save_path or self.DEFAULT_SAVE_PATH
        file_path = Path(save_path)
        if file_path.is_dir():
            filename = f"{to_snake_case(self.title)}.md"
            file_path = file_path / filename
        with file_path.open("w", encoding="utf-8") as f:
            f.write(self.markdown)
        self.save_path = str(file_path.resolve())



from pluggy import HookimplMarker, HookspecMarker
from typing import Optional, Protocol


hookspec = HookspecMarker("mdcli")
hookimpl = HookimplMarker("mdcli")

class ReaderPlugin(Protocol):
    def can_handle(self, url: str) -> bool:
        ...

    def read(self, url: str) -> str:
        ...


@hookspec(firstresult=True)
def get_custom_reader(url: str) -> Optional[ReaderPlugin]:
    """
    Allows plugins to provide a custom reader for a given URL.
    The first plugin that returns a ReaderPlugin instance will be used.
    """
    ...



import re
from urllib.parse import urlparse
from typing import Optional
from bs4 import BeautifulSoup, element, NavigableString


REQUEST_HEADERS = {
    "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36 Edg/126.0.0.0",
    "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
    "Accept-Encoding": "gzip, deflate",
    "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6",
    "Priority": "u=0, i",
    "Sec-Ch-Ua": '"Not/A)Brand";v="8", "Chromium";v="126", "Microsoft Edge";v="126"',
    "Sec-Ch-Ua-Mobile": "?0",
    "Sec-Ch-Ua-Platform": '"macOS"',
    "Sec-Fetch-Dest": "document",
    "Sec-Fetch-Mode": "navigate",
    "Sec-Fetch-Site": "none",
    "Sec-Fetch-User": "?1",
    "Upgrade-Insecure-Requests": "1"
}

class Constants:

    LB_SYMBOL = "[|lb_bl|]"

    ARTICLE_CONTAINERS = [
        ("article", None),
        ("main", None),
        ("body", None)
    ]

    TAGS_TO_CLEAN = [
        lambda el: el.name in ("style", "link", "button", "footer", "header", "aside"),
        lambda el: el.name == "script" and "src" not in el.attrs,
        lambda el: el.name == "script" and "src" in el.attrs and not el.attrs["src"].startswith("https://gist.github.com"),
    ]

    ATTRS_TO_CLEAN = [
        lambda el: 'style' in el.attrs and re.search(r'display\s*:\s*none', el.attrs['style'], re.IGNORECASE),
        lambda el: 'hidden' in el.attrs,
        lambda el: 'class' in el.attrs and 'katex-html' in el.attrs['class'], # katex
    ]

    POST_HANDLERS = [
        lambda el: el.replace(f"{Constants.LB_SYMBOL}{Constants.LB_SYMBOL}", Constants.LB_SYMBOL).replace(Constants.LB_SYMBOL, "\n\n").strip(), # 添加换行使文章更美观
        lambda el: re.sub(r"`\*\*(.*?)\*\*`", r"**`\1`**", el), # 纠正不规范格式 `**code**` 替换为 **`code`**
        lambda el: re.sub(r"`\*(.*?)\*`", r"*`\1`*", el), # 纠正不规范格式 `*code*` 替换为 *`code`*
        lambda el: re.sub(r"`\s*\[([^\]]+)\]\(([^)]+)\)\s*`", r"[`\1`](\2)", el), # 纠正不规范格式 `[code](url)` 替换为 [`code`](url)
        lambda el: re.sub(r"\\\((.+?)\\\)", r"$\1$", el), # 将 \( ... \) 替换为 $ ... $
        lambda el: re.sub(r"\\\[(.+?)\\\]", r"$$\1$$", el), # 将 \[ ... \] 替换为 $$ ... $$
    ]

    INLINE_ELEMENTS = [
        "span", "code", "li", "a", "strong", "em", "img", "b", "i"
    ]

    BLOCK_ELEMENTS = [
        "p", "h1", "h2", "h3", "h4", "h5", "h6", "ul", "ol", "blockquote", "pre", "picture", "hr", "figcaption", "table", "section"
    ]

    TRUSTED_ELEMENTS = INLINE_ELEMENTS + BLOCK_ELEMENTS


def is_sequentially_increasing(code: str) -> bool:
    try:
        # 解码并按换行符拆分
        numbers = [int(line.strip()) for line in code.split('\n') if line.strip()]
        # 检查是否递增
        return all(numbers[i] + 1 == numbers[i + 1] for i in range(len(numbers) - 1))
    except ValueError:
        return False  # 处理非数字情况

def is_block_element(element_name: str) -> bool:
    return element_name in Constants.BLOCK_ELEMENTS

def is_pure_block_children(element: element.Tag) -> bool:
    for child in element.children:
        if isinstance(child, NavigableString):
            if child.strip():  # 有非空文本
                return False
        elif not is_block_element(child.name):
            return False
    return True

def move_spaces(input_string: str, suffix: str) -> str:
    # 使用正则表达式匹配以指定的suffix结尾，且suffix之前有空格的情况
    escaped_suffix = re.escape(suffix)  # 处理正则中的特殊字符
    pattern = rf'(.*?)\s+({escaped_suffix})$'
    match = re.search(pattern, input_string)
    if match:
        # 获取字符串的主体部分（不含空格）和尾部的 '**'
        main_part = match.group(1)
        stars = match.group(2)
        # 计算空格的数量并将空格移动到 '**' 后
        space_count = len(input_string) - len(main_part) - len(stars)
        return f"{main_part}{stars}{' ' * space_count}"
    return input_string

def to_snake_case(input_string: str) -> str:
    input_string = "".join(char if char.isalnum() else " " for char in input_string)
    snake_case_string = "_".join(word.lower() for word in input_string.split())
    return snake_case_string

def collapse_spaces(text) -> str:
    """
    将多个连续空格（包括换行和 Tab）折叠成一个空格。
    """
    return re.sub(r'\s+', ' ', text)

def extract_domain(url: str) -> Optional[str]:
    """
    从URL中提取域名（包含协议）。

    Args:
        url (str): 要提取域名的URL。

    Returns:
        Optional[str]: 提取出的域名（包含协议），如果解析失败或协议不支持则返回 None。
    """
    try:
        parsed_url = urlparse(url)
        if parsed_url.scheme in {"http", "https"} and parsed_url.netloc:
            return f"{parsed_url.scheme}://{parsed_url.netloc}".rstrip('/')
        return None  # 返回 None 表示 URL 格式不符合要求或协议不支持

    except ValueError:
        return None  # 如果 URL 格式无效，则返回 None

def detect_language(file_name: str, code: str) -> str:
    # TODO: 添加语言检测逻辑
    return ''


def extract_article_from_soup(soup: BeautifulSoup, template: tuple) -> element.Tag:
    if template[1] is not None:
        return soup.find(template[0], attrs=template[1])
    else:
        return soup.find(template[0])



import re
from typing import Optional
from bs4 import BeautifulSoup, element, NavigableString
import requests
import importlib
import pkgutil
from pathlib import Path

from .extractor import Article, DefaultExtractor, Extractor
from .utils import (
    Constants,
    is_sequentially_increasing,
    is_block_element,
    move_spaces,
    detect_language,
    collapse_spaces,
    extract_domain,
    is_pure_block_children,
)

class HtmlMarkdownParser:

    def __init__(self, raw_html: str):
        self.raw_html = raw_html
        self.soup = BeautifulSoup(self.raw_html, "html5lib")
        self.extractors = self._load_extractors()
        og_url = self.soup.find("meta", {"property": "og:url"})
        self.url = og_url["content"].strip() if og_url and "content" in og_url.attrs else None

    def parse(self) -> tuple:
        article = self._extract_article()
        if article:
            # print(article)
            markdown = self._process_children(article.body)
            for handler in Constants.POST_HANDLERS:
                markdown = handler(markdown)
            if not article.description or article.description in markdown:
                description = ""
            else:
                description = f"> {article.description}\n\n"
            result = f"# {article.title}\n\n{description}{markdown}"
            # print(result)
            return (article.title, result)
        return (None, None)

    def _process_element(self, element: element.Tag, level: int = 0, is_pre: bool = False) -> str:
        parts = []
        if element.name == "br":
            parts.append(Constants.LB_SYMBOL)
        elif element.name == "hr":
            parts.append("---")
        elif element.name in {"h1", "h2", "h3", "h4", "h5", "h6"}:
            heading = self._process_children(element, level, is_pre=is_pre)
            parts.append(f"{'#' * int(element.name[1])} {heading}")
        elif element.name == "a":
            link = self._process_children(element, level, is_pre=is_pre).replace(Constants.LB_SYMBOL, "")
            if link:
                parts.append(f"[{link}]({element.get("href")})")
        elif element.name == "strong" or element.name == "b":
            parts.append(move_spaces(f"**{self._process_children(element, level, is_pre=is_pre)}**", "**"))
        elif element.name == "em" or element.name == "i":
            parts.append(move_spaces(f"*{self._process_children(element, level, is_pre=is_pre)}*", "*"))
        elif element.name == "ul" or element.name == "ol":
            parts.append(self._process_list(element, level))
        elif element.name == "img":
            src = element.get("data-src") or element.get("src")
            parts.append(self._process_image(src, element.get("alt", "")))
        elif element.name == "blockquote":
            blockquote = self._process_children(element, level, is_pre=is_pre)
            if blockquote.startswith(Constants.LB_SYMBOL):
                blockquote = blockquote.removeprefix(Constants.LB_SYMBOL)
            if blockquote.endswith(Constants.LB_SYMBOL):
                blockquote = blockquote.removesuffix(Constants.LB_SYMBOL)
            parts.append("\n".join(f"> {line}" for line in blockquote.split(Constants.LB_SYMBOL)))
        elif element.name == "pre":
            parts.append(self._process_codeblock(element, level))
        elif element.name == "code": # inner code
            code = self._process_children(element, level, is_pre=is_pre)
            if Constants.LB_SYMBOL not in code:
                parts.append(f"`{code}`")
            else:
                parts.append(code)
        elif element.name == "picture":
            source_elements = element.find_all("source")
            img_element = element.find("img")
            if img_element and source_elements:
                src_set = source_elements[0]["srcset"]
                src = src_set.split()[0]
                parts.append(self._process_image(src, img_element.get("alt", "")))
        elif element.name == "figcaption":
            figcaption = self._process_children(element, level, is_pre=is_pre).replace(Constants.LB_SYMBOL, "\n").strip()
            figcaptions = figcaption.replace("\n\n", "\n").split("\n")
            parts.append("\n".join([f"*{caption}*" for caption in figcaptions]))
        elif element.name == "table":
            parts.append(self._process_table(element, level))
        elif element.name == "math": # 处理latex公式
            semantics = element.find("semantics")
            if semantics:
                tex = semantics.find(attrs={'encoding': 'application/x-tex'})
                if tex:
                    parts.append(f"$$ {tex.text} $$")
        elif element.name == "script": # 处理github gist
            parts.append(self._process_gist(element))
        else:
            parts.append(self._process_children(element, level, is_pre=is_pre))
        result = ''.join(parts)
        if result and is_block_element(element.name):
            if not is_pure_block_children(element):
                result = f"{Constants.LB_SYMBOL}{result}{Constants.LB_SYMBOL}"
        return result

    def _process_children(self, element: element.Tag, level: int = 0, is_pre: bool = False) -> str:
        parts = []
        if element.children:
            # new_level = level + 1 if element.name in Constants.TRUSTED_ELEMENTS else level
            for child in element.children:
                if isinstance(child, NavigableString):
                    if is_pre:
                        parts.append(child)
                    else:
                        result = collapse_spaces(child).replace("<", "&lt;").replace(">", "&gt;")
                        if result.strip():
                            parts.append(result)
                        # print(element.name, level, result)
                else:
                    result = self._process_element(child, level, is_pre=is_pre)
                    if is_pre or len(result.replace(Constants.LB_SYMBOL, "")) != 0:
                        parts.append(result)
        return ''.join(parts) if is_pre or level > 0 else ''.join(parts).strip()

    def _process_list(self, element: element.Tag, level: int) -> str:
        indent = "  " * level
        child_list = element.find_all(recursive=False)
        is_ol = element.name == "ol"
        parts = []
        for i, child in enumerate(child_list):
            if child.name == "li":
                content = self._process_children(child, level).replace(Constants.LB_SYMBOL, "").strip()
                if content:  # 忽略空内容
                    prefix = f"{i + 1}." if is_ol else "-"
                    parts.append(f"{indent}{prefix} {content}")
            elif child.name == "ul" or child.name == "ol":
                content = self._process_element(child, level + 1)
                if content:  # 忽略空内容
                    parts.append(f"{content.replace(Constants.LB_SYMBOL, "")}")
        if not parts:
            return ""  # 所有内容都为空则返回空字符串
        return "\n".join(parts)

    def _process_codeblock(self, element: element.Tag, level: int) -> str:
        # 找出所有 code 标签（可能为 0 个、1 个或多个）
        code_elements = element.find_all("code") or [element]

        # 处理每一个 code 标签并拼接
        code_parts = [
            self._process_children(code_el, level, is_pre=True).replace(Constants.LB_SYMBOL, "\n")
            for code_el in code_elements
        ]
        code = "\n".join(code_parts).strip()

        if is_sequentially_increasing(code):
            return ''  # 忽略行号

        # 尝试提取语言：从第一个 code 标签的 class 中提取 language
        first_code_el = code_elements[0]
        language = next(
            (cls.split('-')[1] for cls in (first_code_el.get("class") or []) if cls.startswith("language-")),
            ""
        )
        if not language:
            language = detect_language(None, code)
        return f"```{language}\n{code}\n```" if language else f"```\n{code}\n```"

    def _process_table(self, element: element.Tag, level: int) -> str:
        if element.find("pre"):
            return self._process_children(element, level)
        # 获取所有行，包括 thead 和 tbody
        rows = element.find_all("tr")
        # 解析表头（如果有）
        headers = []
        if rows and rows[0].find_all("th"):
            headers = [th.get_text(strip=True) for th in rows.pop(0).find_all("th")]
        # 解析表身
        body = [[td.get_text(strip=True) for td in row.find_all("td")] for row in rows]
        # 处理缺失的表头
        if not headers and body:
            headers = body.pop(0)
        # 统一列数
        col_count = max(len(headers), max((len(row) for row in body), default=0))
        headers += [""] * (col_count - len(headers))
        for row in body:
            row += [""] * (col_count - len(row))
        # 生成 Markdown 表格
        markdown_table = []
        markdown_table.append("| " + " | ".join(headers) + " |")
        markdown_table.append("|-" + "-|-".join(["-" * len(h) for h in headers]) + "-|")
        for row in body:
            markdown_table.append("| " + " | ".join(row) + " |")
        return "\n".join(markdown_table)

    def _process_image(self, src: str, alt: str) -> str:
        if src:
            if src.startswith("/") and self.url:
                domain = extract_domain(self.url)
                src = f"{domain}{src}"
            return f"![{alt}]({src})"
        return ""

    def _process_gist(self, element: element.Tag) -> str:
        src = element.attrs["src"]
        pattern = r"/([0-9a-f]+)(?:\.js)?$"
        match = re.search(pattern, src)
        if match:
            gist_id = match.group(1)
        else:
            return ""
        url = f"https://api.github.com/gists/{gist_id}"
        response = requests.get(url)
        response.encoding = "utf-8"
        if response.status_code == 200:
            data = response.json()
            gists = []
            for filename, info in data["files"].items():
                code = info["content"]
                language = detect_language(filename, code)
                gists.append(f"```{language}\n{code}\n```")
            return "\n\n".join(gists)
        else:
            print(f"Fetch gist error: {response.status_code}")
            return ""

    def _extract_article(self) -> Optional[Article]:
        for extract in self.extractors:
            article = extract.extract(self.soup)
            if article:
                return article
        if not article:
            return DefaultExtractor().extract(self.soup)

    def _load_extractors(self, package_name="extractors") -> list[Extractor]:
        extractors_package = Path(__file__).parent / package_name
        extractors = []
        for loader, module_name, is_pkg in pkgutil.iter_modules([extractors_package.resolve()]):
            module = importlib.import_module(f"omni_article_markdown.{package_name}.{module_name}")
            for attr in dir(module):
                cls = getattr(module, attr)
                if isinstance(cls, type) and issubclass(cls, Extractor) and cls is not Extractor:
                    extractors.append(cls())
        return extractors



from bs4 import BeautifulSoup
from ..extractor import Extractor


class MediumExtractor(Extractor):
    """
    Medium
    """

    def __init__(self):
        super().__init__()
        self.attrs_to_clean.extend([
            lambda el: 'data-testid' in el.attrs,
            lambda el: 'class' in el.attrs and 'speechify-ignore' in el.attrs['class'],
        ])

    def can_handle(self, soup: BeautifulSoup) -> bool:
        return self.get_og_site_name(soup) == "Medium"

    def article_container(self) -> tuple:
        return ("article", None)



from bs4 import BeautifulSoup
from ..extractor import Extractor


class ToutiaoExtractor(Extractor):
    """
    今日头条
    """

    def can_handle(self, soup: BeautifulSoup) -> bool:
        title_tag = soup.title
        title = title_tag.text.strip() if title_tag else None
        return title and title.endswith(" - 今日头条")

    def article_container(self) -> tuple:
        return ("div", {"class": "article-content"})



from bs4 import BeautifulSoup
from ..extractor import Extractor


class AppleDevelopExtractor(Extractor):
    """
    Apple Developer Documentation
    """

    def __init__(self):
        super().__init__()
        self.attrs_to_clean.extend([
            lambda el: 'class' in el.attrs and 'eyebrow' in el.attrs['class'],
            lambda el: 'class' in el.attrs and 'platform' in el.attrs['class'],
            lambda el: 'class' in el.attrs and 'title' in el.attrs['class'],
        ])

    def can_handle(self, soup: BeautifulSoup) -> bool:
        return self.get_og_site_name(soup) == "Apple Developer Documentation"

    def article_container(self) -> tuple:
        return ("main", {"class": "main"})



from bs4 import BeautifulSoup
from ..extractor import Extractor


class FreediumExtractor(Extractor):
    """
    freedium.cfd
    """

    def can_handle(self, soup: BeautifulSoup) -> bool:
        title_tag = soup.title
        title = title_tag.text.strip() if title_tag else None
        return title and title.endswith(" - Freedium")

    def article_container(self) -> tuple:
        return ("div", {"class": "main-content"})

    def extract_title(self, soup: BeautifulSoup) -> str:
        title_tag = soup.find("h1")
        title = title_tag.text.strip()
        title_tag.decompose()
        return title

    def extract_description(self, soup: BeautifulSoup) -> str:
        description_tag = soup.find("h2")
        description = description_tag.text.strip()
        description_tag.decompose()
        return description



from bs4 import BeautifulSoup
from ..extractor import Extractor


class HugoExtractor(Extractor):
    """
    Hugo博客
    """

    def can_handle(self, soup: BeautifulSoup) -> bool:
        return False

    def article_container(self) -> tuple:
        return ("div", {"class": "post-content"})
